
#include <stdio.h>
#include <string.h>
#include <tap.h>
#include "exTap.h"
#include "tf5800.h"
#include "ad_skipper_43.h"

//-----------------------------------------------------------------------------
//
//keep endian friendly
//file is in big endian order
//toppy is big endian, pc is little endian
//
int load_cutfile(char *cut_filename, type_section *section)
{
    TYPE_File *Ip;
    dword flen;
    byte *fbuf;
    int ret;
    dword len;
    dword type;
    dword length;
    byte *fend;
    char rec_filename[0x100];
    int i;

//TAP_Print("\nload_cutfile(cut_filename=%s)", cut_filename);
    //create .rec filename
    len = strlen(cut_filename);
    memcpy(&rec_filename[0], cut_filename, len-4);
    memcpy(&rec_filename[len-4], ".rec\0", 5);

//don't create twice
    if (section->info.filename[0]) {
        if (strcmp(section->info.filename, rec_filename) != 0) {
            //got the wrong one
            create_section(section);
        }
        else {
            //got the right one
            return 0;
        }
    }
    else {
        //none loaded
        create_section(section);
//        strcpy(section->info.filename, rec_filename);
//        section->info.length = strlen(section->info.filename) + 0x11;
    }

    ret = TAP_Hdd_ChangeDir("/");
    ret = TAP_Hdd_ChangeDir("DataFiles");
    if (ret == 0) {
        sprintf(strLog, "error: cannot cd to /DataFiles");
        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_MEDIUM);
        return -1;
    }
    ret = TAP_Hdd_ChangeDir("CutFiles");
    if (ret == 0) {
        TAP_Hdd_Create("CutFiles", ATTR_FOLDER);
    }
    Ip = TAP_Hdd_Fopen(cut_filename);
    if (Ip == 0) {
        strcpy(section->info.filename, rec_filename);
        section->info.length = strlen(section->info.filename) + 0x11;
        section->info.scanned = 0;
        section->info.analysed = 0;
        TAP_Print("\nnote: TAP_Hdd_Fopen(%s), Ip = %x", cut_filename, Ip);
        return 1;
    }
    flen = TAP_Hdd_Flen(Ip);
    if (flen < 0x200) flen = 0x200;
    fbuf = (byte *)malloc(flen);
    ret = TAP_Hdd_Fseek(Ip, 0, SEEK_SET);
    len = TAP_Hdd_Fread(fbuf, 1, flen, Ip);
    TAP_Hdd_Fclose(Ip);

    fend = fbuf + len;
    while (fbuf < fend) {
        type = to_be32(*(dword *)fbuf);
        fbuf += sizeof(dword);
        length = to_be32(*(dword *)fbuf);
        fbuf += sizeof(dword);
        switch(type) {
            case CUT_SECTION_INFO:
                section->info.type = type;
                section->info.length = length;
                memcpy(section->info.filename, fbuf, length-0x10);
                fbuf += (length-0x10);
                section->info.loaded = to_be32(*(dword *)fbuf);
                fbuf += sizeof(dword);
                section->info.scanning = to_be32(*(dword *)fbuf);
                fbuf += sizeof(dword);
                section->info.scanned = to_be32(*(dword *)fbuf);
                fbuf += sizeof(dword);
                section->info.analysed = to_be32(*(dword *)fbuf);
                fbuf += sizeof(dword);
                break;
            case CUT_SECTION_ASPECT:
                section->aspect.type = type;
                section->aspect.length = length;
                section->aspect.count = to_be32(*(dword *)fbuf);
                fbuf += sizeof(dword);
                section->aspect.cut = (type_position *)malloc(section->aspect.count * sizeof(type_position));
//                memcpy(section->aspect.cut, fbuf, section->aspect.count * sizeof(type_position));
//                fbuf += section->aspect.count * sizeof(type_position);
                for (i=0; i<(int)section->aspect.count; i++) {
                    section->aspect.cut[i].from = to_be32(*(dword *)fbuf);
                    fbuf += sizeof(dword);
                    section->aspect.cut[i].to = to_be32(*(dword *)fbuf);
                    fbuf += sizeof(dword);
                    *(dword *)(0+(int)&section->aspect.cut[i].fpos64) = to_be32(*(dword *)&fbuf[0]);
                    *(dword *)(4+(int)&section->aspect.cut[i].fpos64) = to_be32(*(dword *)&fbuf[4]);
                    fbuf += sizeof(ulong64);
                }
                break;
            case CUT_SECTION_IFRAME:
                section->iframe.type = type;
                section->iframe.length = length;
                section->iframe.min_length = to_be32(*(dword *)fbuf);
                fbuf += sizeof(dword);
                *(dword *)(0+(int)&section->iframe.min_fpos64) = to_be32(*(dword *)&fbuf[0]);
                *(dword *)(4+(int)&section->iframe.min_fpos64) = to_be32(*(dword *)&fbuf[4]);
                fbuf += sizeof(ulong64);
                section->iframe.count = to_be32(*(dword *)fbuf);
                fbuf += sizeof(dword);
                section->iframe.ilength = (type_length *)malloc(section->iframe.count * sizeof(type_length));
//                memcpy(section->iframe.ilength, fbuf, section->iframe.count * sizeof(type_length));
//                fbuf += section->iframe.count * sizeof(type_length);
                for (i=0; i<(int)section->iframe.count; i++) {
                    section->iframe.ilength[i].length = to_be32(*(dword *)fbuf);
                    fbuf += sizeof(dword);
                    *(dword *)(0+(int)&section->iframe.ilength[i].fpos64) = to_be32(*(dword *)&fbuf[0]);
                    *(dword *)(4+(int)&section->iframe.ilength[i].fpos64) = to_be32(*(dword *)&fbuf[4]);
                    fbuf += sizeof(ulong64);
                }
                break;
            case CUT_SECTION_GFRAME:
                section->gframe.type = type;
                section->gframe.length = length;
                section->gframe.min_length = to_be32(*(dword *)fbuf);
                fbuf += sizeof(dword);
                *(dword *)(0+(int)&section->gframe.min_fpos64) = to_be32(*(dword *)&fbuf[0]);
                *(dword *)(4+(int)&section->gframe.min_fpos64) = to_be32(*(dword *)&fbuf[4]);
                fbuf += sizeof(ulong64);
                section->gframe.count = to_be32(*(dword *)fbuf);
                fbuf += sizeof(dword);
                section->gframe.ilength = (type_length *)malloc(section->gframe.count * sizeof(type_length));
//                memcpy(section->gframe.ilength, fbuf, section->gframe.count * sizeof(type_length));
//                fbuf += section->gframe.count * sizeof(type_length);
                for (i=0; i<(int)section->gframe.count; i++) {
                    section->gframe.ilength[i].length = to_be32(*(dword *)fbuf);
                    fbuf += sizeof(dword);
                    *(dword *)(0+(int)&section->gframe.ilength[i].fpos64) = to_be32(*(dword *)&fbuf[0]);
                    *(dword *)(4+(int)&section->gframe.ilength[i].fpos64) = to_be32(*(dword *)&fbuf[4]);
                    fbuf += sizeof(dword);
                }
                break;
            default:
                if ((type & 0xffff0000) == CUT_SECTION_MASK) {
                    TAP_Print("\nunhandled section");
                    fbuf += length;
                }
                else {
                    //abort
                    goto end;
                }
                break;
        }
end:
        ;
    }

    free(fbuf);
    return 0;
}



//
//keep endian friendly
//file is in big endian order
//toppy is big endian, pc is littler endian
//
int save_cutfile(type_section *section, char *cut_filename)
{
    TYPE_File *Op;
    dword flen;
    byte *fbuf;
    int ret;
    dword len;
    byte *fstart;
    int i;

//    TAP_Print("\nsave_cutfile()");
    ret = TAP_Hdd_ChangeDir("/");
    ret = TAP_Hdd_ChangeDir("DataFiles");
    if (ret == 0) {
        sprintf(strLog, "error: cannot cd to /DataFiles");
        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_MEDIUM);
        return -1;
    }
    ret = TAP_Hdd_ChangeDir("CutFiles");
    if (ret == 0) {
        TAP_Hdd_Create("CutFiles", ATTR_FOLDER);
    }
    if (TAP_Hdd_Exist(cut_filename)) {
        TAP_Print("\n%s exists, delete it", cut_filename);
        TAP_Hdd_Delete(cut_filename);
    }
    TAP_Hdd_Create(cut_filename, ATTR_NORMAL);
    Op = TAP_Hdd_Fopen(cut_filename);
    ret = TAP_Hdd_Fseek(Op, 0, SEEK_SET);

    flen = 0x4000;                                       //!!!
    fstart = (byte *)malloc(flen);
    fbuf = fstart;

    len = (strlen(section->info.filename) + 4) & 0xfffffffc;   //dword align
    section->info.length = len + 0x10;
    *(dword *)fbuf = to_be32(section->info.type);
    fbuf += sizeof(dword);
    *(dword *)fbuf = to_be32(section->info.length);
    fbuf += sizeof(dword);
    memcpy(fbuf, section->info.filename, len);
    fbuf += len;
    *(dword *)fbuf = to_be32(section->info.loaded);
    fbuf += sizeof(dword);

    *(dword *)fbuf = to_be32(section->info.scanning);
    fbuf += sizeof(dword);
    *(dword *)fbuf = to_be32(section->info.scanned);
    fbuf += sizeof(dword);
    *(dword *)fbuf = to_be32(section->info.analysed);
    fbuf += sizeof(dword);

    section->aspect.length = 0x04 + section->aspect.count * sizeof(type_position);
    *(dword *)fbuf = to_be32(section->aspect.type);
    fbuf += sizeof(dword);
    *(dword *)fbuf = to_be32(section->aspect.length);
    fbuf += sizeof(dword);
    *(dword *)fbuf = to_be32(section->aspect.count);
    fbuf += sizeof(dword);
//    memcpy(fbuf, section->aspect.cut, section->aspect.count * sizeof(type_position));
//    fbuf += section->aspect.length;
    for (i=0; i<(int)section->aspect.count; i++) {
        *(dword *)fbuf = to_be32(section->aspect.cut[i].from);
        fbuf += sizeof(dword);
        *(dword *)fbuf = to_be32(section->aspect.cut[i].to);
        fbuf += sizeof(dword);
        *(dword *)&fbuf[0] = to_be32(*(dword *)(0+(int)&section->aspect.cut[i].fpos64));
        *(dword *)&fbuf[4] = to_be32(*(dword *)(4+(int)&section->aspect.cut[i].fpos64));
        fbuf += sizeof(ulong64);
    }

    section->iframe.length = 0x0c + section->iframe.count * sizeof(type_length);
    *(dword *)fbuf = to_be32(section->iframe.type);
    fbuf += sizeof(dword);
    *(dword *)fbuf = to_be32(section->iframe.length);
    fbuf += sizeof(dword);
    *(dword *)fbuf = to_be32(section->iframe.min_length);
    fbuf += sizeof(dword);
    *(dword *)&fbuf[0] = to_be32(*(dword *)(0+(int)&section->iframe.min_fpos64));
    *(dword *)&fbuf[4] = to_be32(*(dword *)(4+(int)&section->iframe.min_fpos64));
    fbuf += sizeof(ulong64);
    *(dword *)fbuf = to_be32(section->iframe.count);
    fbuf += sizeof(dword);
//    memcpy(&fbuf, section->iframe.ilength, section->iframe.count * sizeof(type_length));
//    fbuf += section->iframe.length;
    for (i=0; i<(int)section->iframe.count; i++) {
        *(dword *)fbuf = to_be32(section->iframe.ilength[i].length);
        fbuf += sizeof(dword);
        *(dword *)&fbuf[0] = to_be32(*(dword *)(0+(int)&section->iframe.ilength[i].fpos64));
        *(dword *)&fbuf[4] = to_be32(*(dword *)(4+(int)&section->iframe.ilength[i].fpos64));
        fbuf += sizeof(ulong64);
    }

    section->gframe.length = 0x0c + section->gframe.count * sizeof(type_length);
    *(dword *)fbuf = to_be32(section->gframe.type);
    fbuf += sizeof(dword);
    *(dword *)fbuf = to_be32(section->gframe.length);
    fbuf += sizeof(dword);
    *(dword *)fbuf = to_be32(section->gframe.min_length);
    fbuf += sizeof(dword);
    *(dword *)&fbuf[0] = to_be32(*(dword *)(0+(int)&section->gframe.min_fpos64));
    *(dword *)&fbuf[4] = to_be32(*(dword *)(4+(int)&section->gframe.min_fpos64));
    fbuf += sizeof(ulong64);
    *(dword *)fbuf = to_be32(section->gframe.count);
    fbuf += sizeof(dword);
//    memcpy(fbuf, section->gframe.ilength, section->gframe.count * sizeof(type_length));
//    fbuf += section->gframe.length;
    for (i=0; i<(int)section->gframe.count; i++) {
        *(dword *)fbuf = to_be32(section->gframe.ilength[i].length);
        fbuf += sizeof(dword);
        *(dword *)&fbuf[0] = to_be32(*(dword *)(0+(int)&section->gframe.ilength[i].fpos64));
        *(dword *)&fbuf[4] = to_be32(*(dword *)(4+(int)&section->gframe.ilength[i].fpos64));
        fbuf += sizeof(ulong64);
    }

    flen = fbuf-fstart;
    TAP_Print("\nwriting %x bytes", flen);
    TAP_Hdd_Fwrite(fstart, 1, flen, Op);
    TAP_Hdd_Fclose(Op);
    return 0;
}

//-----------------------------------------------------------------------------
int create_section(type_section *section)
{
//    TAP_Print("\ncreate_section()");
    //don't create twice
    if (section->info.filename[0]) {
        free_section(section);
    }
    section->info.type = CUT_SECTION_INFO;
    section->info.length = 0x11;
    section->info.filename[0] = 0;
    section->info.loaded   = 0;
    section->info.scanning = 0;
    section->info.scanned  = 0;
    section->info.analysed = 0;

    section->aspect.type = CUT_SECTION_ASPECT;
    section->aspect.length = 0x04 + 0x100 * sizeof(type_position);
    section->aspect.count = 0;
    section->aspect.cut = (type_position *)malloc(0x100 * sizeof(type_position));

    section->iframe.type = CUT_SECTION_IFRAME;
    section->iframe.length = 0x0c + 0x100 * sizeof(type_length);
    section->iframe.min_length = 0x7fffffff;
    section->iframe.min_fpos64 = 0x7fffffffffffffff;
    section->iframe.count = 0;
    section->iframe.ilength = (type_length *)malloc(0x100 * sizeof(type_length));

    section->gframe.type = CUT_SECTION_GFRAME;
    section->gframe.length = 0x0c + 0x100 * sizeof(type_length);
    section->gframe.min_length = 0x7fffffff;
    section->gframe.min_fpos64 = 0x7fffffffffffffff;
    section->gframe.count = 0;
    section->gframe.ilength = (type_length *)malloc(0x100 * sizeof(type_length));

    return 0;
}

void free_section(type_section *section)
{
//    TAP_Print("\nfree_section()");
    section->info.filename[0] = 0;
    section->info.loaded   = 0;
    section->info.scanning = 0;
    section->info.scanned  = 0;
    section->info.analysed = 0;
//    free(section->info.filename);
    free(section->aspect.cut);
    free(section->iframe.ilength);
    free(section->gframe.ilength);
}

void dump_section(type_section *section)
{
    int i;

//    TAP_Print("\ndump_section()");
    TAP_Print("\ninfo.filename     = %s", section->info.filename);
    TAP_Print("\ninfo.loaded       = %x", section->info.loaded);
    TAP_Print("\ninfo.scanning     = %x", section->info.scanning);
    TAP_Print("\ninfo.scanned      = %x", section->info.scanned);
    TAP_Print("\ninfo.analysed     = %x", section->info.analysed);
    TAP_Print("\naspect.count      = %x", section->aspect.count);
    for (i=0; i<(int)section->aspect.count; i++) {
        TAP_Print("\n  %x%08x: from %s to %s", *(dword *)(0+(int)&section->aspect.cut[i].fpos64), *(dword *)(4+(int)&section->aspect.cut[i].fpos64),
            (section->aspect.cut[i].from==2) ? "4:3" : (section->aspect.cut[i].from==3) ? "16:9" : "??:?",
            (section->aspect.cut[i].to==2)   ? "4:3" : (section->aspect.cut[i].to==3)   ? "16:9" : "??:?"
        );
    }
//    TAP_Print("\niframe.min_length = %x", section->iframe.min_length);
//    TAP_Print("\niframe.min_fpos   = %x", section->iframe.min_fpos);
//    TAP_Print("\niframe.count      = %x", section->iframe.count);
//    for (i=0; i<section->iframe.count; i++) {
//        TAP_Print("\n  %x%08x: length %x", *(dword *)(0+(int)&section->iframe.ilength[i].fpos64), *(dword *)(4+(int)&section->iframe.ilength[i].fpos64), section->iframe.ilength[i].length);
//    }
//    TAP_Print("\ngframe.min_length = %x", section->gframe.min_length);
//    TAP_Print("\ngframe.min_fpos   = %x", section->gframe.min_fpos);
//    TAP_Print("\ngframe.count      = %x", section->gframe.count);
//    for (i=0; i<section->gframe.count; i++) {
//        TAP_Print("\n  %x%08x: length %x", *(dword *)(0+(int)&section->gframe.ilength[i].fpos64), *(dword *)(4+(int)&section->gframe.ilength[i].fpos64), section->gframe.ilength[i].length);
//    }

}

